# @(#)05  1.10 src/cmdline/staging/scripts/fed_utils.py, amws.cmdline, fim611, 070402a 3/28/07 16:48:47
# (C) Copyright International Business Machines Corporation 2005. All rights
# reserved.  U.S. Government Users Restricted Rights -- Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM

import fim_base, configuration_parameters
import saml20_utils
from com.tivoli.am.fim.middlemanager.util import MgmtUtils
from com.tivoli.am.fim.middlemanager.ssps import Federation, FederationManager, ConfigurationParameters, Partner
from com.tivoli.am.fim.middlemanager.tokenservice import SecurityTokenManager
from java.util import HashMap
from java.lang import Integer
from com.tivoli.am.fim.logging import FIMLoggerFactory
from com.tivoli.am.fim.logging import FIMLogger
from com.tivoli.am.fim.logging import FIMLevel
from com.tivoli.am.fim.i18n.msgs import CmdLineMessages
from jarray import array
from java.lang import Object
from com.tivoli.am.fim.logging import FIMLoggerConstants

# Set up FIMLogger
component_name = "StagingTools"
script_name = "fed_utils.py"
method_name = script_name
class_name = "com.tivoli.am.fim.cmdline." + script_name[0:-3]
FIMLoggerFactory.enableStdout(FIMLevel.ALL)
log = FIMLoggerFactory.getFIMLogger(component_name, class_name, CmdLineMessages.CLASS_NAME)
FIMLoggerFactory.setPreferredFIMLoggerImpl(FIMLoggerConstants.FIMLOGGER_BASE_IMPL)

########
# Lower level utilities
#
def create_succinct_id(id_str):
    sha = MgmtUtils.getSHA1Hash(id_str)
    return MgmtUtils.base64Encode(sha)
    
def create_base_endpoint_prefix(endpt_base, display_name):
    return     "%s/%s"%(endpt_base,display_name)
    
def read_file_to_string(file_name):
    f = open(file_name, 'r')
    s = f.read()
    f.close()
    return s

def read_file_to_bytes(filename):
    f = open(filename, 'rb')
    bytes = f.read()
    f.close()
    return bytes
        
########
# create_federations
#
# Takes a Jython dictionary object mapping a ConfigurationParameters object for 
# a federation to a Jython list of ConfigurationParameters objects for
# federation partners.  It uses the given FederationManager (fed_mgr) object
# and SecurityTokenManager (sec_tok_mgr) object to create the federation
# and its partners
#
########
def create_federations(fed_map, fed_mgr, sec_tok_mgr):
    fed_cfgs = fed_map.keys()
    for fed_cfg in fed_cfgs:
        fed = fed_mgr.createFederation(fed_cfg, sec_tok_mgr)
        partner_cfgs = fed_map[fed_cfg]
        for ptnr_cfg in partner_cfgs:
            ptnr = Partner(fed, ptnr_cfg)             
            fed.addPartner(ptnr)
            ptnr.activate()
            
    # Commit changes
    fim_base.commit_session(fed_mgr)
    
#########
# delete_federations
#
# Takes a Jython list of EntityGroup (federation) UUID's
# and removes all information associated with them from
# the server, using the provided FederationManager (fed_mgr)
# object and SecurityTokenManager (sec_tok_mgr) object.
##########
def delete_federations(fed_list, fed_mgr, sec_tok_mgr):    
    for fed_id in fed_list:
        fed = fed_mgr.getFederation(fed_id, sec_tok_mgr)
        fed_mgr.deleteFederation(fed)
        
    # Commit changes
    fim_base.commit_session(fed_mgr)
        
#########
# get_federations
#
# Given a FederationManager instance and a SecurityTokenManager instance
# return a dictionary of federation display names and ID's
#
##########
def get_federations(fed_mgr):
    fed_id_map = {}
    fed_list = fed_mgr.listAllFederationDescriptors()

    for fed_desc in fed_list:
        fed_id = fed_desc.getId()
        fed_name = fed_desc.getDisplayName()
        fed_id_map[fed_id] = fed_name
    return fed_id_map

#########
# get_partners
#
# Given a Federation object return a dictionary of partner display names and ID's
#
##########
def get_partners(fed):
    partner_id_map = {}
    for partner in fed.getPartnerList():
        partner_name = partner.getCfgParameters().getDisplayName()
        partner_id = partner.getPartnerId()
        partner_id_map[partner_id] = partner_name
    return partner_id_map

    
#########
# get_federation
#
# Takes the federation name and the path to the properties file
# and returns the appropriate federation object
#########
 
def get_federation(fed_name, domain_props_file, inst_name="domain"):
    # Get domain managment objects
    domain_props = fim_base.read_properties_file(domain_props_file)
    (fed_mgr, sec_tok_mgr) = fim_base.prepare_mgr_objs(domain_props, inst_name)    

    if fed_mgr.hasFederationDisplayName(fed_name):
        # Return federation object
        return (fed_mgr.getFederationByDisplayName(fed_name, sec_tok_mgr), fed_mgr)
    else:
        return (None,fed_mgr)


#########
# get_token_properties
#
# Given a federation or partner object, return a dictionary containing
# token properties and their associated values.
##########

def get_token_properties(obj):
# The 'obj' argument to get_token_properties() is either a Federation or 
# a Partner object
    method_name = "get_token_properties"
    
    obj_cfg = obj.getCfgParameters()
    tok_map = HashMap()
    tok_map = obj_cfg.getTokenInstanceProperties()

    tok_dict={}
    if not tok_map.isEmpty():
        keys = tok_map.keySet()    
        for key in keys:
            tok_dict[key] = tok_map.get(key)
    return tok_dict

#########
# get_federation_properties
#
# This function retrieves the properties of a federation or a partner 
# based on the role and protocol.  The logic for getting the properties
# is contained in the configuration_parameters.py and the saml20_utils.py files.
#
# Input is either a federation, or a federation and a partner object.
#    If both a federation and partner are given, then the partner properties
#    for the specified partner in that federation are returned
# Output is a dictionary of property/values, where property is the string
#    used in the staging tools properties files.  If both a federation and 
#    partner are given, he partner properties for the specified partner in 
#    that federation are returned.
# 
##########

def get_federation_properties(fed, partner=None):
    # Get federation configuration
    fed_role = fed.getSelfRoleStr()
    fed_protocol = fed.getSSOProtocolStr()
    if partner:
        isPartner = fim_base.FIM_TRUE
        cfg = partner.getCfgParameters()
        # Get the token properties for this partner
        token_props = get_token_properties(partner)
    else:
        isPartner = fim_base.FIM_FALSE
        cfg = fed.getCfgParameters()
        # Get the token properties for this federation
        token_props = get_token_properties(fed)
    
    # methods is a dictionary of property/method pairs, which
    # maps the property string found in the staging tools configuration files
    # to a FIM API get method
    methods = configuration_parameters.get_methods(fed_role, fed_protocol,\
        isPartner, "get")
    
    # Create a new dictionary containing property name/value pairs
    fed_props = {}
    for key in methods.keys():
        # cfg.getDisplayName() returns the Company Name, rather
        # than the display name, so we must use fed.getDisplayName()
        # All other federation properties are got from cfg.get<Method>()
        if key == "DisplayName":
            if not isPartner:
                fed_props[key] = fed.getDisplayName()
                continue
            
        # Populate the fed_props dictionary by executing 
        # the ConfigurationParameter get methods
        value = eval("cfg.%s()"%methods[key])
        if value:
            fed_props[key] = eval("cfg.%s()"%methods[key])
        elif value==0:
            fed_props[key]=int(0)
        else:
            fed_props[key] = ""

    # Get the dictionary that maps the keys in token_props to the 
    # human readable form found in the staging tools properties files
    token_prop_dict = configuration_parameters.get_token_dictionary(fed_role, \
        fed_protocol, isPartner)

    # Add token props to the fed_props dictionary
    for key in token_props.keys():
        prop_key = token_prop_dict[key] # prop_key is human readable
        fed_props[prop_key] = token_props[key] # add to the dictionary started above
    
    # Add SAML 2.0 profile propeties (i.e. endpoints) to the fed_props dictionary
    if fed_protocol=="SAML2_0":
        saml20_svc_props = saml20_utils.get_saml20_profile_properties(cfg, fed_role, isPartner)
        for key in saml20_svc_props.keys():
            fed_props[key] = saml20_svc_props[key]

    return fed_props

#########
# validate_user_properties
#
# Validates the user-supplied properties against the set of properties
# supported for the particular federation role (IP or SP) and protocol.
##########
    
def validate_user_properties(user_props, valid_props, fed_role, fed_protocol):
    method_name = "validate_user_properties"

    for key in user_props.keys():
        if key not in valid_props:
            inserts = array([key, fed_role, fed_protocol], Object)
            log.logMessage(FIMLevel.INFO, method_name, \
                CmdLineMessages.invalid_user_property, inserts)
            raise SystemExit
    return
    
#########
# set_federation_properties
#
# This function sets the properties of a federation or a partner 
# based on the role and protocol.  The logic for getting the properties
# is contained in the configuration_parameters.py and the saml20_utils.py files.
#
# Input is a federation object, a user properties dictionary and optionally 
#    a partner object.
#    fed = the federation object to be modified.
#    user_props = a dictionary of federation/partner properties to 
#        change for this federation or partner
#    partner = the partner object to be modified.  If this option is
#        given, then the function will update the partner object
#        of the given federation
# 
##########

def set_federation_properties(fed, user_props, partner=None):
    method_name = "set_federation_properties"
    
    # Get federation configuration
    fed_role = fed.getSelfRoleStr()
    fed_protocol = fed.getSSOProtocolStr()
    if partner:
        isPartner = fim_base.FIM_TRUE
        cfg = partner.getCfgParameters()
    else:
        isPartner = fim_base.FIM_FALSE
        cfg = fed.getCfgParameters()

    # methods is a dictionary of property/method pairs, which
    # maps the property string found in the staging tools configuration files
    # to a get method
    methods = configuration_parameters.get_methods(fed_role, fed_protocol,\
        isPartner, "set")

    # Get the dictionary that maps the keys in token_props to the 
    # human readable form found in the staging tools properties files
    token_prop_dict = configuration_parameters.get_token_dictionary(fed_role, \
        fed_protocol, isPartner)
    
    # get the SAML 2.0 profile property keys
    saml20_profiles = saml20_utils.get_saml20_supported_profiles(fed_role, isPartner)

    # validate that user-supplied properties match the supported 
    # properties for the given role/protocol combination 
    all_props = methods.keys() + token_prop_dict.values() + saml20_profiles
    validate_user_properties(user_props, all_props, fed_role, fed_protocol)
    
    # All values in user_props start off as strings, so strings like "100" that 
    # are supposed to represent integers need to be converted to integers
    # with the int() function.  eval() aids in the conversion of the other
    # types.  If rhs is a PyString, eval() reduces it to a variable, which 
    # evaluates causes a NameError exception for FIM properties.
    #
    # Jython syntax has been adopted for the format for multi-valued properties
    # in the user-specified configuration files.  It has the following syntax:
    # for a PyList or a PyDictionary, e.g
    #    prop1 = ["v1", "v2"]
    # or
    #    prop2 = {"k1":"v1", "k2":"v2"}
    token_properties = HashMap()
    for key in user_props.keys():
        try:
            rhs = eval(user_props[key])
            if str(type(rhs)).find("PyInteger") > 0:
                value = int(rhs)
            elif str(type(rhs)).find("PyList") > 0:
                value = rhs
                user_props[key] = rhs
            elif str(type(rhs)).find("PyDictionary") > 0:
                value = rhs
                user_props[key] = rhs
            else:
                value=str(user_props[key])
        except NameError:
            value = str(user_props[key])
        except SyntaxError:
            value = str(user_props[key])

        if methods.has_key(key):
        # cfg.setDisplayName() returns the Company Name, rather
        # than the display name, so we must use fed.getDisplayName()
        # All other federation properties are set by cfg.set<Method>()
            if key == "DisplayName":
                if isPartner:
                    code = fim_base.create_set_code(methods[key])
                    fim_base.run_set_code(code, cfg, value)
                else:
                    fed.setDisplayName(value)
            else:
                code = fim_base.create_set_code(methods[key])
                fim_base.run_set_code(code, cfg, value)
        elif token_prop_dict.values().count(key):
            token_key = fim_base.get_key_from_value(token_prop_dict, key)
            token_properties.put(token_key, str(value))

    # Re-calculate the succinct IDs
    if fed_protocol=="SAML2_0":
        cfg.setSourceId(create_succinct_id(cfg.getProviderId()))
        # Set the SAML 2.0 profile endpoints
        saml20_utils.set_saml20_profile_properties(cfg, fed_role, isPartner, user_props)
    elif fed_protocol=="SAML1_1" or fed_protocol=="SAML1_0":
        cfg.setSuccinctId(create_succinct_id(cfg.getProviderId()))
    elif fed_protocol=="LBTY1_1" or fed_protocol=="LBTY1_2":
        cfg.setSuccinctProviderId(create_succinct_id(cfg.getProviderId()))
                                                                                
    # Set the token properties
    if not token_properties.isEmpty():
        cfg.setTokenInstanceProperties(token_properties)
        
    # IY96453
    # Remove syntax error from script and always set federation state,
    # to ensure the correct update of properties.
    fed.modifyState()
    # IY96453

    return 

#######
# list_federations
#
# Given a FederationManager instance and a SecurityTokenManager instance 
# print out the list of federations by name or ID and name
#
########

def list_federations(fed_mgr, show_id = None):
    fed_id_map=get_federations(fed_mgr)
    if show_id:
        feds = fed_id_map.keys()
        feds.sort()
        for fed in feds:
            print fed + ",  " + fed_id_map[fed]
    else:
        feds = fed_id_map.values()
        feds.sort()
        for fed in feds:
            print fed
            
    return

#######
# list_partners
#
# Given a Federation object print out the list of partners by name and/or ID
#
########

def list_partners(fed, show_id = None):
    partner_id_map = get_partners(fed)
    if show_id:
        partners = partner_id_map.keys()
        partners.sort()
        for partner in partners:
            print partner + ",  " + partner_id_map[partner]
    else:
        partners = partner_id_map.values()
        partners.sort()
        for partner in partners:
            print partner
    return

#######
# show_federation
#
# Print out the properties of a given federation
#
########

def show_federation(fed):
    fed_props = get_federation_properties(fed)
    fim_base.print_dictionary(fed_props) # print out the properties dictionary

#######
# show_partner
#
# Print out the properties of the given partner of the given federation
#
########

def show_partner(fed, partner):
    part_props = get_federation_properties(fed, partner)
    fim_base.print_dictionary(part_props) # print out the properties dictionary


#######
# import_federation
#
# This function is designed to move federation properties from one
# FIM domain to another (i.e. from a "source" domain to a "target"
# domain).  The inputs are:
#    src_fed = the federation object from the source domain.
#    targ_fed_mgr = the federation manager from the target domain
#    targ_sec_tok_mgr = the federation manager from the target domain
#    user_props = a dictionary of federation/partner properties to 
#        change for the target domain
#    fed_name = a string representing the name of the federation to import
#    import_partners = a boolean indicating whether all the partners
#        should be imported as well.  Note that all partners will be
#        imported as is.  If you want to modify partner properties,
#        then set this to FIM_FALSE and use the import_partner function.
#########

def import_federation(src_fed, targ_fed_mgr, targ_sec_tok_mgr, user_props, fed_name,
        import_partners=None):
    
    method_name = "import_federation"

    # Check to see if federation already exists 
    if targ_fed_mgr.hasFederationDisplayName(fed_name):
        inserts = array([fed_name], Object)
        log.logMessage(FIMLevel.INFO, method_name, \
            CmdLineMessages.federation_already_exists, inserts)
        raise SystemExit

    # Create federation in target domain using the source federation configuration
    src_fed_cfg = src_fed.getCfgParameters()
    # Create the federation with the user-supplied name, if present
    if user_props.has_key("DisplayName"):
        fed_name = user_props["DisplayName"]
    targ_fed = targ_fed_mgr.createFederation(fed_name, src_fed_cfg, targ_sec_tok_mgr)
    src_description = src_fed.getDescription()
    targ_fed.setDescription(src_description)

    if not targ_fed_mgr.hasFederationDisplayName(fed_name):
        inserts = array([fed_name], Object)
        log.logMessage(FIMLevel.WARNING, method_name, \
            CmdLineMessages.unable_to_import_federation, inserts)
        raise SystemExit
    
    # Update the imported federation with the user-specified configuration changes
    set_federation_properties(targ_fed, user_props)

    # Import all of the partners from the source domain without modification
    if import_partners:
        partners = src_fed.getPartnerList()
        for partner in partners:
            targ_fed.addPartner(partner)

    targ_fed.modifyState()
    
    # Commit changes
    fim_base.commit_session(targ_fed_mgr)

    return

#######
# get_partner
#
# Given a federation and a partner name, return the partner object
########

def get_partner(fed, partner_name):
    partners = fed.getPartnerList()
    for partner in partners:
        partner_cfg = partner.getCfgParameters()
        display_name = partner_cfg.getDisplayName()
        if display_name == partner_name:
            return partner
    return None

#######
# import_partner
#
# This function is designed to move partner properties from one
# FIM domain to another (i.e. from a "source" domain to a "target"
# domain).  The inputs are:
#    src_fed = the federation object from the source domain.
#    targ_fed = the federation object from the target domain.
#    user_props = a dictionary of partner properties to 
#        change for the target domain
#    partner_name = a string representing the name of the federation to import
#########

def import_partner(src_fed, targ_fed, user_props, partner_name):
    method_name = "import_partner"
    
    partner = get_partner(src_fed, partner_name)
    if partner:
        set_federation_properties(src_fed, user_props, partner)
        targ_fed.addPartner(partner) # Add partner to the target federation
        targ_fed.modifyState()
    else:
        inserts = array([partner_name, src_fed.getDisplayName()], Object)
        log.logMessage(FIMLevel.INFO, method_name, \
            CmdLineMessages.partner_does_not_exist, inserts)
        raise SystemExit
                  
    return

